float4x4 modelMatrix;
float4x4 modelMatrixInverse;
float4x4 modelViewProjMatrix;

struct IaosTrafo
{
    float4 pos	: POSITION;
    float3 normal 	: NORMAL;
    float2 tex 	: TEXCOORD;
};

struct VsosTrafo
{
    float4 pos 		: SV_POSITION;
    float4 worldPos 	: WORLDPOS;
    float3 normal 		: NORMAL;
    float2 tex 		: TEXCOORD;
};

VsosTrafo vsTrafo(IaosTrafo input)
{
	VsosTrafo output = (VsosTrafo)0;
	output.pos = mul(input.pos,
		modelViewProjMatrix);
	output.worldPos = mul(input.pos,
		modelMatrix);
	output.normal = mul(modelMatrixInverse,
		float4(input.normal.xyz, 0.0));
	output.tex = input.tex;
	return output;
}

Texture2D kdTexture;

SamplerState linearSampler
{
	Filter = MIN_MAG_MIP_LINEAR;
    AddressU = wrap;
    AddressV = wrap;
	BorderColor = float4(1, 0, 1, 1);
};

float3 eyePos;
TextureCube envTexture;

float4 psExam(VsosTrafo input) : SV_Target
{
	return float4(1, 0, 0, 1);
}

struct IaosQuad
{
	float4  pos: POSITION;
	float2  tex: TEXCOORD0;
};

struct VsosQuad
{
	float4 pos: SV_POSITION;
	float2 tex: TEXCOORD0;
	float3 viewDir: TEXCOORD1;
};

float4x4 viewDirMatrix;

VsosQuad vsQuad(IaosQuad input)
{
	VsosQuad output = (VsosQuad)0;
    output.pos = input.pos;
    float4 hWorldPosMinusEye = mul(input.pos, viewDirMatrix);
//    hWorldPosMinusEye /= hWorldPosMinusEye.w;
    output.viewDir = hWorldPosMinusEye.xyz;
    output.tex = input.tex;
    return output;
}

float4 psBackground(VsosQuad input) : SV_Target
{
	return envTexture.Sample(linearSampler, input.viewDir);
}

technique11 exam
{
	pass exam
	{
        SetVertexShader ( CompileShader( vs_5_0, vsTrafo() ) );
        SetPixelShader( CompileShader( ps_5_0, psExam() ) );
 	}
	pass background
	{
        SetVertexShader ( CompileShader( vs_5_0, vsQuad() ) );
        SetPixelShader( CompileShader( ps_5_0, psBackground() ) );
 	}
}